mutable struct ScanningStrategy{T<:AbstractFloat, I<:Int, AA<:AbstractArray{T}, AS<:AbstractString}
    nside::I
    times::I
    sampling_rate::I
    alpha::T
    beta::T
    prec_period::T
    spin_rpm::T
    hwp_rpm::T
    FP_theta::AA
    FP_phi::AA
    start_point::AS
end

@inline function initial_pointings(ss::ScanningStrategy)
    if ss.start_point =="pole"
        #= Set the initial position of the boresight to near the zenith. =#
        initial_vec = @views @SVector [sind(ss.alpha-ss.beta), 0, cosd(ss.alpha-ss.beta)]
    end
    if ss.start_point == "equator"
        #= Set the initial position of the boresight to near the equator. =#
        initial_vec = @views @SVector [cosd(ss.alpha-ss.beta), 0 , sind(ss.alpha-ss.beta)]
    end
    initial_vec
end

#=
@inline function rotate_by_quarternion!(target_quart, quart1, quart2)
    #q_theta_in_FP = quaternion_rotator(deg2rad(SS.FP_theta[j]), 1, y_axis)
    #q_phi_in_FP = quaternion_rotator(deg2rad(SS.FP_phi[j]), 1, bore_0)
    #q_for_FP = quart1 * quart1
    return quart2 * quart1 * target_quart / quart1 / quart2
end
=#

@inline function get_pointings(SS::ScanningStrategy, start::Int, stop::Int)
    resol = Resolution(SS.nside)

    omega_spin = (2π / 60) * SS.spin_rpm
    omega_prec = (2π / 60) / SS.prec_period
    omega_revol = (2π) / (60.0 * 60.0 * 24.0 * 365.25)
    
    #= Compute the TOD of the hitpixel and the TOD of the detector orientation at a specified sampling rate from time start to stop. =#
    time_array = @views Vector(start:1/SS.sampling_rate:stop-1/SS.sampling_rate)
    loop_times = length(time_array)
    
    psi_tod = @views zeros(loop_times, length(SS.FP_theta))
    theta_tod = @views zeros(loop_times, length(SS.FP_theta))
    phi_tod = @views zeros(loop_times, length(SS.FP_theta))
    
    #psi_tod = @views zeros(Float32, loop_times, length(SS.FP_theta))
    #ang_tod = @views zeros(Float32, 2, loop_times, length(SS.FP_theta))
    
    antisun_axis = @views @SVector [1.0, 0.0, 0.0]
    y_axis = @views @SVector [0.0, 1.0, 0.0]
    z_axis = @views @SVector [0.0, 0.0, 1.0]
    spin_axis = @views @SVector [cosd(SS.alpha), 0.0, sind(SS.alpha)]
    
    
    bore_0 = initial_pointings(SS)
    
    #= Generate the quaternion at the initial position of the boresight and detector orientation. =#
    boresight_0 = @views Quaternion(0.0, bore_0)
    detector_vec_0 = @views @SVector [-boresight_0.q3, 0.0, boresight_0.q1]
    detector_orientation_0 = @views Quaternion(0.0, detector_vec_0)
    travel_direction_vec_0 = @views bore_0 × detector_vec_0
    travel_direction_0 = @views Quaternion(0.0, travel_direction_vec_0)

    @views @inbounds @simd for j = eachindex(SS.FP_theta)
        #= 
        Generating the pointhing quaternion of other detectors in the focal plane 
        by adding an angular offset to the boresight based on the SS.FP_theta and SS.FP_phi information. 
        =#
        
        q_theta_in_FP = quaternion_rotator(deg2rad(SS.FP_theta[j]), 1, y_axis)
        q_phi_in_FP = quaternion_rotator(deg2rad(SS.FP_phi[j]), 1, bore_0)
        q_for_FP = q_phi_in_FP * q_theta_in_FP
        pointing = q_for_FP * boresight_0 / q_for_FP
        
        #pointing = @views rotate_by_quarternion!(boresight_0, quaternion_rotator(deg2rad(SS.FP_theta[j]), 1, y_axis), quaternion_rotator(deg2rad(SS.FP_phi[j]), 1, bore_0))
        
        @views @inbounds @threads for i = eachindex(time_array)
            t = @views time_array[i]
            #= Generate the quaternion of revolution, precession, and spin at time t. =#
            q_revol = quaternion_rotator(omega_revol, t, z_axis)
            q_prec = quaternion_rotator(omega_prec, t, antisun_axis)
            q_spin = quaternion_rotator(omega_spin, t, spin_axis)
            #= 
            P has the value of the pointing vector (x,y,z) at time t as its imaginary part.
            The direction that the detector orientation is facing is then calculated as det_vec_t.
            =#
            Q = q_revol * q_prec * q_spin
            P = Q * pointing / Q
            q_det = Q * detector_orientation_0 / Q
            travel_direction = Q * travel_direction_0 / Q
            pointing_t = @SVector [P.q1, P.q2, P.q3]
            travel_direction_vec = @SVector [travel_direction.q1, travel_direction.q2, travel_direction.q3]
            #=
            The direction of movement can be calculated by the outer product of the pointhing vector and the detector orientaion vector.
            The vector of meridians can be calculated by combining the outer product of pointing and z axis.
            =#
            longitude = pointing_t × (pointing_t × z_axis)
            ang_t = vec2ang_ver2(pointing_t[1], pointing_t[2], pointing_t[3])

            theta_tod[i, j] = @views ang_t[1]
            phi_tod[i, j] = @views ang_t[2]
            
            #pix_tod[i, j] = ang2pixRing(resol, ang_t[1], ang_t[2])
            #pix_tod[i, j] = ang2pix(_m_, bore_ang[1], bore_ang[2])
            
            #=
            The vector standing perpendicular to the sphere is defined as the outer product of the meridian and the moving_direction. 
            The direction of divergent_vec depends on whether the trajectory enters the sphere from the left or the right side of the meridian.
            =#
            divergent_vec = longitude × travel_direction_vec
            cosK = dot(travel_direction_vec, longitude) / (norm(travel_direction_vec) * norm(longitude))
            #=
            An if statement to prevent errors due to rounding errors that may result in |cosK|>1
            =#
            cosK = ifelse(abs(cosK) > 1.0, sign(cosK), cosK)
            
            psi_tod[i, j] = acos(cosK) * sign(divergent_vec[3]) * sign(pointing_t[3])
        end
    end
    return (theta_tod, phi_tod, psi_tod, time_array)
end


@inline function get_pointing_pixels(SS::ScanningStrategy, start::Int, stop::Int)
    #SS = @views ScanningStrategyStructure
    resol = @views Resolution(SS.nside)

    omega_spin = @views (2π / 60) * SS.spin_rpm
    omega_prec = @views (2π / 60) / SS.prec_period
    omega_revol = @views (2π) / (60.0 * 60.0 * 24.0 * 365.25)
    
    #= Compute the TOD of the hitpixel and the TOD of the detector orientation at a specified sampling rate from time start to stop. =#
    time_array = @views Vector(start:1/SS.sampling_rate:stop-1/SS.sampling_rate)
    loop_times = length(time_array)
    
    pix_tod = @views zeros(Int64, loop_times, length(SS.FP_theta))
    psi_tod = @views zeros(loop_times, length(SS.FP_theta))
    #ang_tod = @views zeros(Float32, 2, loop_times, length(FP_theta))
    
    antisun_axis = @views @SVector [1.0, 0.0, 0.0]
    y_axis = @views @SVector [0.0, 1.0, 0.0]
    z_axis = @views @SVector [0.0, 0.0, 1.0]
    spin_axis = @views @SVector [cosd(SS.alpha), 0.0, sind(SS.alpha)]
    
    bore_0 = initial_pointings(SS)
    
    #= Generate the quaternion at the initial position of the boresight and detector orientation. =#
    boresight_0 = @views Quaternion(0.0, bore_0)
    detector_vec_0 = @views @SVector [-boresight_0.q3, 0.0, boresight_0.q1]
    detector_orientation_0 = @views Quaternion(0.0, detector_vec_0)
    travel_direction_vec_0 = @views bore_0 × detector_vec_0
    travel_direction_0 = @views Quaternion(0.0, travel_direction_vec_0)

    @views @inbounds @simd for j = eachindex(SS.FP_theta)
        #= 
        Generating the pointhing quaternion of other detectors in the focal plane 
        by adding an angular offset to the boresight based on the FP_theta and FP_phi information. 
        =#
        q_theta_in_FP = quaternion_rotator(deg2rad(SS.FP_theta[j]), 1, y_axis)
        q_phi_in_FP = quaternion_rotator(deg2rad(SS.FP_phi[j]), 1, bore_0)
        q_for_FP = q_phi_in_FP * q_theta_in_FP
        pointing = q_for_FP * boresight_0 / q_for_FP
        @views @inbounds @threads for i = eachindex(time_array)
            
            t = @views time_array[i]
            #= Generate the quaternion of revolution, precession, and spin at time t. =#
            q_revol = quaternion_rotator(omega_revol, t, z_axis)
            q_prec = quaternion_rotator(omega_prec, t, antisun_axis)
            q_spin = quaternion_rotator(omega_spin, t, spin_axis)
            #= 
            P has the value of the pointing vector (x,y,z) at time t as its imaginary part.
            The direction that the detector orientation is facing is then calculated as det_vec_t.
            =#
            Q = q_revol * q_prec * q_spin
            P = Q * pointing / Q
            q_det = Q * detector_orientation_0 / Q
            travel_direction = Q * travel_direction_0 / Q
            pointing_t = @SVector [P.q1, P.q2, P.q3]
            travel_direction_vec = @SVector [travel_direction.q1, travel_direction.q2, travel_direction.q3]
            #=
            The direction of movement can be calculated by the outer product of the pointhing vector and the detector orientaion vector.
            The vector of meridians can be calculated by combining the outer product of pointing and z axis.
            =#
            longitude = pointing_t × (pointing_t × z_axis)
            ang_t = vec2ang_ver2(pointing_t[1], pointing_t[2], pointing_t[3])

            #ang_tod[1, i, j] = ang_t[1]
            #ang_tod[2, i, j] = ang_t[2]
            
            pix_tod[i, j] = ang2pixRing(resol, ang_t[1], ang_t[2])
            #pix_tod[i, j] = ang2pix(_m_, bore_ang[1], bore_ang[2])
            
            #=
            The vector standing perpendicular to the sphere is defined as the outer product of the meridian and the moving_direction. 
            The direction of divergent_vec depends on whether the trajectory enters the sphere from the left or the right side of the meridian.
            =#
            divergent_vec = longitude × travel_direction_vec
            cosK = dot(travel_direction_vec, longitude) / (norm(travel_direction_vec) * norm(longitude))
            #=
            An if statement to prevent errors due to rounding errors that may result in |cosK|>1
            =#
            cosK = ifelse(abs(cosK) > 1.0, sign(cosK), cosK)
            psi_tod[i, j] = acos(cosK) * sign(divergent_vec[3]) * sign(pointing_t[3])
        end
    end
    return (pix_tod, psi_tod, time_array)
end
